from modeller import *
from modeller.automodel import *

env = environ()
a = automodel(env, alnfile='TvLDH-1bdmA.ali',
              knowns='1bdmA', sequence='TvLDH',
              assess_methods=(assess.DOPE, assess.GA341))
a.starting_model = 1
a.ending_model = 5
a.make()



from modeller import *

log.verbose()
env = environ()

#-- Prepare the input files

#-- Read in the sequence database
sdb = sequence_db(env)
sdb.read(seq_database_file='pdb_95.pir', seq_database_format='PIR',
         chains_list='ALL', minmax_db_seq_len=(30, 4000), clean_sequences=True)

#-- Write the sequence database in binary form
sdb.write(seq_database_file='pdb_95.bin', seq_database_format='BINARY',
          chains_list='ALL')

#-- Now, read in the binary database
sdb.read(seq_database_file='pdb_95.bin', seq_database_format='BINARY',
         chains_list='ALL')

#-- Read in the target sequence/alignment
aln = alignment(env)
aln.append(file='TvLDH.ali', alignment_format='PIR', align_codes='ALL')

#-- Convert the input sequence/alignment into
#   profile format
prf = aln.to_profile()

#-- Scan sequence database to pick up homologous sequences
prf.build(sdb, matrix_offset=-450, rr_file='${LIB}/blosum62.sim.mat',
          gap_penalties_1d=(-500, -50), n_prof_iterations=1,
          check_profile=False, max_aln_evalue=0.01)

#-- Write out the profile in text format
prf.write(file='build_profile.prf', profile_format='TEXT')

#-- Convert the profile back to alignment format
aln = prf.to_alignment()

#-- Write out the alignment file
aln.write(file='build_profile.ali', alignment_format='PIR')




import re
import os
from loader2 import *
#arq = #nome do arquivo


class pdb2(pdb):
#  string file, chains;
    def __init__(self, arq):
        self.nomedoarq = arq
        self.file = file(arq, 'r')
        self.FileInList = self.file.readlines()


    
    def head(self):
        newFileInList = ""
        for i in self.FileInList:
            if not i[0:6]==("HETATM") and not i.startswith("ATOM") and not i.startswith("MASTER") and not i.startswith("TER") and not i.startswith("END") and not i.startswith("CONECT"):
                newFileInList = newFileInList + i
        return(newFileInList)
    def atom(self, cadeia, inicio):
        newFileInList = ""
        NumeroLinha = inicio+1
        for linha in self.FileInList:                
            if linha.startswith("ATOM") and linha[21] == cadeia:
                LinhaNova = linha[0:4]+" "*(7-len(str(NumeroLinha)))+str(NumeroLinha)+linha[11:(len(linha))]
                newFileInList = newFileInList + LinhaNova
                NumeroLinha = NumeroLinha+1                
        return(newFileInList);
        
    def hetatom(self, cadeia, inicio):
        newFileInList = ""
        NumeroLinha = inicio +1
        for linha in self.FileInList:
            if linha[0:6]==("HETATM") and linha[21] == cadeia:
                #print NumeroLinha
                LinhaNova = linha[0:6]+" "*(5-len(str(NumeroLinha)))+str(NumeroLinha)+linha[11:(len(linha))] #5
                newFileInList = newFileInList + LinhaNova
                NumeroLinha = NumeroLinha+1
        return(newFileInList);
    def fim(self, cadeia, inicio):
        newFileInList = ""
        NumeroLinha = inicio +1
        for linha in self.FileInList:
            if linha.startswith("TER") and linha[21] == cadeia:
                LinhaNova = linha[0:3]+" "*(8-len(str(NumeroLinha)))+str(NumeroLinha)+linha[11:(len(linha))]
                newFileInList = newFileInList + LinhaNova
                NumeroLinha = NumeroLinha+1
        return(newFileInList);    
    def conects(self):
        newFileInList = ""
        for linha in self.FileInList:
            if linha.startswith("CONECT"):
                newFileInList = newFileInList + linha
        return(newFileInList)
    def masters(self):
        newFileInList = ""
        for linha in self.FileInList:
            if linha.startswith("MASTER"):
                newFileInList = newFileInList + linha
        return(newFileInList)
    def ends(self):
        newFileInList = ""
        for linha in self.FileInList:
            if linha.startswith("END"):
                newFileInList = newFileInList + linha
        return(newFileInList)
    def construir(self):        
        newFileInList = self.head()
        inicio = 0
        
        for cadeia in self.chains():
            if newFileInList.splitlines()[-1][0:6]==("HETATM")  or newFileInList.splitlines()[-1].startswith("ATOM")   or newFileInList.splitlines()[-1].startswith("TER"): 
           # if newFileInList[-1].startswith("ATOM"):
                inicio = int(newFileInList.splitlines()[-1][6:11])
            else:
                inicio = 0
            newFileInList = newFileInList + self.atom(cadeia, inicio)
            inicio = int(newFileInList.splitlines()[-1][6:11])
            newFileInList = newFileInList + self.hetatom(cadeia, inicio)
            inicio = int(newFileInList.splitlines()[-1][6:11])
            newFileInList = newFileInList + self.fim(cadeia, inicio)
        inicio = int(newFileInList.splitlines()[-1][6:11])
        newFileInList = newFileInList + self.hetatom(" ", inicio)
        newFileInList = newFileInList + self.conects()
        newFileInList = newFileInList + self.masters()
        newFileInList = newFileInList + self.ends()
        
            #atoms
            #hetatoms
            #fim
        #hetatoms sem cadeia
        #fim
        return(newFileInList)

def fixpdb(arquivo):
  fix = pdb2(arquivo)
  arquivotemp = file(arquivo + ".tmp","w")
  arquivotemp.write(fix.construir())
  arquivotemp.close()
  os.rename(arquivo + ".tmp", arquivo)
#a = pdb2("1j36.pdb")
#print a.atom("A",0)
#x = file("/joao/Testes/teste.txt","w")
#x.write(a.construir())
#x.close()
#fixpdb("1j36.pdb")




import os

class Procheck_Evaluate:
    def __init__(self,workdir,template):
        self.workdir = workdir
        self.procheck_workdir = self.prepare_folder()
        self.template = self.workdir + os.path.basename(template)
        self.template_on_procheck_folder = None
    def prepare_folder(self):
        pastadeavaliacao = self.workdir + 'avaliacao_procheck/'
        if os.path.exists(pastadeavaliacao):
	        os.popen('rm -fr '+ pastadeavaliacao)
        os.mkdir(pastadeavaliacao)
        return pastadeavaliacao

    def copy_templates_for_procheck_evaluate(self):
        os.popen('cp '+ self.template +  ' ' + self.procheck_workdir)
        self.template_on_procheck_folder = self.procheck_workdir + os.path.basename(self.template)

    def run_prochek(self):
      script = """\
import os
#path = os.getcwd()
os.chdir('""" + self.procheck_workdir + """')
os.popen('procheck '+ '""" + self.template_on_procheck_folder + """' +  ' ' + str(2.0))
#os.chdir(path)
      """
      script_path = self.procheck_workdir + "procheck.py"
      procheck_script = file(script_path, "w")        
      procheck_script.write(script)
      procheck_script.close()
      os.popen("python " + script_path)
      return self.procheck_workdir[0:-1] 


import pdb, os
from Modeller_Caller import modeller_caller
from RunCommand import RunCommand
from Bio import SearchIO
from AutomodelServerModules.AlignmentBuilder import AlignmentBuilder
class Align:
    def __init__(self,pdb_folder, pfam_database,modeldir,pdb_file,seq_ali):
        self.seq_ali = modeldir + seq_ali
        self.pfam_database = pfam_database
        self.pdb_folder = pdb_folder
        self.modeldir = modeldir
        self.pdb_file = pdb.pdb(modeldir + pdb_file)
        self.template = modeldir + pdb_file
        self.strname = self.pdb_file.NomedaEstrutura()

        self.hmmresults = None
        self.betterid = None
        self.hmm = None
        self.fasta_aligned_file = None
        self.template_pir = None


    def convert_seqali_pir_to_fasta_formar(self):
        script = """\
from modeller import *
import os
os.chdir('""" + self.__folder_of_model__() + """')
env = environ()
env.io.hetatm = env.io.water = False
env.io.atom_files_directory = ['""" + self.__folder_of_model__() + """']
code = '""" + self.strname + """'   #   estrutura a ser lida#
mdl = model(env, file=code)

aln = alignment(env)
aln.append_model(mdl, align_codes=code)
aln.write(file='""" + self.__folder_of_model__() + """/template.pir', alignment_format = 'PIR')
aln.write(file='""" + self.__folder_of_model__() + """/template.fasta', alignment_format = 'FASTA')

aln2 = alignment(env)
aln2.append(file='""" + self.__folder_of_model__() + """/seq.ali', alignment_format = 'PIR')
aln2.write(file='""" + self.__folder_of_model__() + """/seq.fasta', alignment_format='FASTA')

unalinedfasta = file('""" + self.__folder_of_model__() + """/unaligned.fasta', 'w')
templatefasta = file('""" + self.__folder_of_model__() + """/template.fasta', 'r')
seqfasta = file('""" + self.__folder_of_model__() + """/seq.fasta', 'r')

unalinedlist = templatefasta.readlines()
unalinedlist = unalinedlist + seqfasta.readlines()

unalinedfasta.writelines("".join(unalinedlist))
unalinedfasta.close()
"""
        arq = file(self.modeldir + 'get_fasta.py', 'w')
        arq.write(script)
        arq.close()

        self.unaligned_seq = self.__folder_of_model__() + '/unaligned.fasta'
        # self.template_pir = self.__folder_of_model__() + '/template.pir'
        processo = modeller_caller()
        processo.run(self.__folder_of_model__() + 'get_fasta.py')

        self.__get_template_sequence_in_pir_format__()


    def __get_template_sequence_in_pir_format__(self):
        script = """\
from modeller import *
import os
os.chdir('""" + self.__folder_of_model__() + """')
env = environ()
env.io.hetatm = env.io.water = True
env.io.atom_files_directory = './:../atom_files'

code = '""" + self.strname + """'   #   estrutura a ser lida#
mdl = model(env, file=code)

aln = alignment(env)
aln.append_model(mdl, align_codes=code)
aln.write(file='""" + self.__folder_of_model__() + """/str.seq', alignment_format = 'PIR')
"""
        arq = file(self.modeldir + 'get_template.py', 'w')
        arq.write(script)
        arq.close()

        # self.unaligned_seq = self.__folder_of_model__() + '/unaligned.fasta'
        self.template_pir = self.__folder_of_model__() + '/str.seq'
        processo = modeller_caller()
        processo.run(self.__folder_of_model__() + 'get_template.py')

    def search_an_hmmm(self):
          seqfasta = self.modeldir + "/seq.fasta"
          result =  self.modeldir + "/hmmsearch.txt"
          hmmer = RunCommand(["hmmsearch",self.pfam_database, seqfasta, ">", result])
          hmmer.run()
          # print "foi"
          # seqfasta = modeldir + "/seq.fasta"
          # result =  modeldir + "/hmmsearch.txt"
          # os.popen("hmmsearch " + self.pfam_database + " " + seqfasta + " > " + result)
          self.hmmresults = result

    def find_better_motif(self):
      better_result = None
      for qresults in SearchIO.parse(self.hmmresults,'hmmer3-text'):
          if(len(qresults.hits)>0 and better_result != None):
            print qresults.hits
            print qresults.seq_len
            print better_result.seq_len
            if qresults.seq_len > better_result.seq_len:
              print qresults.seq_len
              better_result = qresults
          else:
            if len(qresults.hits)>0:
              better_result = qresults
      print better_result.id
      self.betterid =  better_result.id

    def fetch_the_hmm(self):
          result = self.modeldir + "PRF.hmm"
          hmmer = RunCommand(["hmmfetch",self.pfam_database, self.betterid,">", result])
          hmmer.run()
          self.hmm =  result

    def align_with_hmmer(self):
        result = self.modeldir + "seq.ali"
        hmmer = RunCommand(["hmmalign", self.hmm, self.unaligned_seq ,">", result])
        hmmer.run()
        self.fasta_aligned_file = result

    def align_with_muscle(self):
      result = self.modeldir + "seq.ali"
      muscle = RunCommand(["muscle -in ", self.unaligned_seq ," -out ", result])
      muscle.run()
      self.fasta_aligned_file = result

    def convert_fasta_to_pir(self):
      # print self.template
      # print self.fasta_aligned_file
      # print self.template_pir
      try:
        x = AlignmentBuilder(self.modeldir, os.path.basename(self.template), self.fasta_aligned_file,  self.template_pir)
        print "---"
        aliali = self.__folder_of_model__() + 'ali.ali'
        alingFile = file(aliali,'w')
        alingFile.write(x.__get_template_sequence__() + "\n\n") 
        alingFile.write(x.__get_target_sequence__())
        alingFile.close()
        self.aliali = aliali
      except Exception, e:
        print e

    def __folder_of_model__(self):
        return self.modeldir


from modeller import *

env = environ()
aln = alignment(env)
mdl = model(env, file='1bdm', model_segment=('FIRST:A','LAST:A'))
aln.append_model(mdl, align_codes='1bdmA', atom_files='1bdm.pdb')
aln.append(file='TvLDH.ali', align_codes='TvLDH')
aln.align2d()
aln.write(file='TvLDH-1bdmA.ali', alignment_format='PIR')
aln.write(file='TvLDH-1bdmA.pap', alignment_format='PAP')



import os
class RunCommand:
    def __init__(self, parameters):
        # self.command = [command]
        self.parameters = parameters
        self.process = None
        # fullcommand = self.modeller_executable  + self.parameters
    def run(self):
        # try:
            
            os.popen(" ".join(self.parameters))
            # self.process = subprocess.Popen(self.parameters)
        # except:
        #     print "Modeller or Script not found!"
        # return self.process.wait()




class PirFormat(object):
	"""docstring for PirFormat"""

	def __init__(self, name, type, description, start, chain_start, stop, chain_stop, sequence):
		self.name = name
		self.type = type
		self.description = description
		self.start = start
		self.chain_start = chain_start
		self.stop = stop
		self.chain_stop = chain_stop
		self.sequence = sequence

	def get(self):
		return ">P1;" + self.name + "\n" + self.type + ":" + self.name + ":" + self.start + ":" \
		 + self.chain_start + ":" + self.stop + ":" + self.chain_stop + ":" + ":" \
		 + self.description + ":" + ":" + "\n" + self.sequence + "*"
		


from Modeller_Caller import modeller_caller
import os
class GetProt(object):
    """docstring for GetProt"""
    def __init__(self, loop_folder):
        self.myscript = ""
        self.path = loop_folder

    def create_script_in_folder(self, template,model,myloop):
        template = os.path.basename(template)[0:-4]
        model = os.path.basename(model)[0:-4]
        myloop = os.path.basename(myloop)[0:-4]
        # self.path = os.path.dirname(model)
        script = """import matplotlib.pyplot as plt
import numpy as np
import modeller
import os

os.chdir('""" + self.path + """')
def get_profile(profile_file, seq):
    '''Read `profile_file` into a Python array, and add gaps corresponding to
       the alignment sequence `seq`.'''
    # Read all non-comment and non-blank lines from the file:
    f = file(profile_file)
    vals = []
    for line in f:
        if not line.startswith('#') and len(line) > 10:
            spl = line.split()
            vals.append(float(spl[-1]))
    # Insert gaps into the profile corresponding to those in seq:
    for n, res in enumerate(seq.residues):
        for gap in range(res.get_leading_gaps()):
            vals.insert(n, None)
    # Add a gap at position '0', so that we effectively count from 1:
    vals.insert(0, None)
    return vals

e = modeller.environ()
a = modeller.alignment(e, file='ali.ali')

try:
    template = get_profile('""" + template + """.profile', a['""" + template + """A'])
    model = get_profile('""" + model + """.profile', a['""" + model + """'])
    myloop  = get_profile('""" + myloop + """.profile', a['""" + myloop + """'])
except Exception, e:
        
    template = get_profile('""" + template + """.profile', a['""" + template + """'])
    model = get_profile('""" + model + """.profile', a['seq.ali'])
    myloop  = get_profile('""" + myloop + """.profile', a['seq.ali'])


# Plot the template and model profiles in the same plot for comparison:
fig = plt.figure(2)
ax = fig.add_subplot(111)
plt.xlabel('Alignment position')
plt.ylabel('DOPE per-residue score')
ax.plot(model, color='red', linewidth=2, label='Model')
ax.plot(template, color='green', linewidth=2, label='Template')
ax.plot(myloop, color='blue', linewidth=2, label='Loop refinement model')
handles, labels = ax.get_legend_handles_labels()
lgd = ax.legend(handles, labels, loc='upper center', bbox_to_anchor=(0.5,-0.1))
ax.grid('on')
fig.savefig('dope_profile_loop', bbox_extra_artists=(lgd,), bbox_inches='tight')
"""
        script_path = self.path + os.sep + "plot_profile.py"
        loop_script = file(script_path, "w")        
        loop_script.write(script)
        loop_script.close()
        self.myscript = script_path

    def get_model(self):
        processo = modeller_caller()
        processo.run(self.myscript)



from modeller import *
from modeller.scripts import complete_pdb

log.verbose()    # request verbose output
env = environ()
env.libs.topology.read(file='$(LIB)/top_heav.lib') # read topology
env.libs.parameters.read(file='$(LIB)/par.lib') # read parameters

# directories for input atom files
env.io.atom_files_directory = './:../atom_files'

# read model file
mdl = complete_pdb(env, '1bdm.pdb', model_segment=('FIRST:A', 'LAST:A'))

s = selection(mdl)
s.assess_dope(output='ENERGY_PROFILE NO_REPORT', file='1bdmA.profile',
              normalize_profile=True, smoothing_window=15)



from modeller import *

env = environ()
aln = alignment(env)
for (pdb, chain) in (('1b8p', 'A'), ('1bdm', 'A'), ('1civ', 'A'),
                     ('5mdh', 'A'), ('7mdh', 'A'), ('1smk', 'A')):
    m = model(env, file=pdb, model_segment=('FIRST:'+chain, 'LAST:'+chain))
    aln.append_model(m, atom_files=pdb, align_codes=pdb+chain)
aln.malign()
aln.malign3d()
aln.compare_structures()
aln.id_table(matrix_file='family.mat')
env.dendrogram(matrix_file='family.mat', cluster_cut=-1.0)



import re
import os
class pdb:
#  string file, chains;
    def __init__(self, arq):
        self.nomedoarq = arq
        self.file = file(arq, 'r')
        self.FileInList = self.file.readlines()        
    def Nome(self):
        return(self.file.name)
    def NomedaEstrutura(self):
        PrimeiraLinha = self.FileInList[0]
        if "HEADER" in PrimeiraLinha:
            return PrimeiraLinha[62:66].lower()
        else:
            print("Excecao: Arquivo PDB fora do Padrao")
    def nomedoarquivo(self):
        return(self.nomedoarq)
    def chains(self): #inutilizado
        List = []
        for i in self.LinhasOndeTemEscritoChain():
            texto = self.FileInList[i]
            if "CHAIN:" in texto:
                ChainPositionInTheString = texto.find(":") + 2
                if ";" in texto:
                    EndPositionInTheString = texto.find(";")
                else:
                    EndPositionInTheString = len(texto)-1                
                for j in range(ChainPositionInTheString,  EndPositionInTheString):
                    if texto[j] not in [' ',',']:
                        List.append(texto[j])            
               
            else:
                #excecao
                print("error")
        return(List)
    def LinhasOndeTemEscritoChain(self):
        Linhas = []
        Contador = 0
        for i in self.FileInList: 

            if "CHAIN:" in i:
                Linhas.append(Contador)
            Contador+=1 
        return(Linhas)
    
    def hetatm(self):
        for i in self.FileInList:
            if  "HETATM" in i:
                return(True)
        return(False)
        
    def hoh(self):
       if self.hetatm():
            for i in self.FileInList:
                if "HOH" in i:
                    return(True) 
            return(False)
       else: 
           return(False)
    
    def HetatomsInPDB(self):
        if self.hetatm():
            List = []
            for i in self.FileInList:
                if ("HETATM" in i) and not ("REMARK" in i):
                    #Line = filter(None, re.split(" ", i))
                    if not i[17:20].replace(" ", "") in List:
                        List.append(i[17:20].replace(" ", ""))
            return(List)
        else:
            print("excecao")
            
    def ChangeHetatoms(self, Cadeia, Hetoriginal, Hetnovo):
        pdbsaida = ''
        if self.hetatm():
            for linha in self.FileInList:
                if ("HETATM" in linha) and not ("REMARK" in linha) :
                    #Line = filter(None, re.split(" ", linha))
                    if  (linha[17:20].replace(" ", "") == Hetoriginal) and ( linha[21] in Cadeia):
                        if len(Hetnovo) == 2:
                            Hetnovo = Hetnovo + " "
                        pdbsaida = pdbsaida + linha.replace(Hetoriginal,  Hetnovo)
                    else:
                        pdbsaida =  pdbsaida +linha
                else:
                        pdbsaida =  pdbsaida +linha
            return(pdbsaida)
                            

    
    def const(self,  hetatms,  chains):
        #if chains in self.chains() and hetatms in self.HetatomsInPDB():
            pdbsaida = ''
            for i in self.FileInList:
                Line = filter(None, re.split(" ", i))

                if(Line[0] != "CONECT"):
                    if ((Line[0] == "COMPND") and (Line[2] == "CHAIN:")):
                        texto = "COMPND   3 CHAIN: "
                        for letra in range(0, len(chains)):
                            texto = texto + chains[letra]
                            if (letra != len(chains)-1):
                                texto = texto + ","                        
                        pdbsaida = pdbsaida + texto + ";" + (62-2*len(chains))*(" ") + "\n"
                        #pdbsaida = pdbsaida 

                    else:
                        if ((i.startswith("ATOM")) or (i.startswith("HETATM"))):
                                 if((i[21] in chains) and (i[17:20].replace(" ", "") in hetatms[i[21]])):
                                    # pdbsaida = pdbsaida + "\n" + i
                                    pdbsaida = pdbsaida  + i
                                 else:
                                     pass
                                 if((i.startswith("ATOM")) and (i[21] in chains)):
                                    pdbsaida = pdbsaida + i
                                 else:
                                    pass
                        else:
                                # pdbsaida = pdbsaida + "\n" + i
                                pdbsaida = pdbsaida + i
            return(pdbsaida)

#    def const(self,  hetatms,  chains):
#        #if chains in self.chains() and hetatms in self.HetatomsInPDB():
#            pdbsaida = ''
#            for i in self.FileInList:
#                Line = filter(None, re.split(" ", i))
##                    if(Line[1] == "COMPND") and (Line[3] == "CHAIN:"):
#                        #Mudar cadeias
#                if(Line[0] != "CONECT"):
#                    if ((Line[0] == "COMPND") and (Line[2] == "CHAIN:")):
#                        texto = "COMPND   3 CHAIN: "
#                        for letra in range(0, len(chains)):
#                            texto = texto + chains[letra]
#                            if (letra != len(chains)-1):
#                                texto = texto + ","                        
#                        pdbsaida = pdbsaida + texto + ";" + (62-2*len(chains))*(" ") + "\n"
#                        #pdbsaida = pdbsaida 
#
#                    else:
#                        if ((Line[0] == "ATOM") or (Line[0] ==  "HETATM")):
#                                 if((Line[4][0] in chains) and (Line[3] in hetatms[Line[4][0]])):
#                                    # pdbsaida = pdbsaida + "\n" + i
#                                    pdbsaida = pdbsaida  + i
#                                 else:
#                                     pass
#                                 if((Line[0] == "ATOM") and (Line[4][0] in chains)):
#                                    pdbsaida = pdbsaida + i
#                                 else:
#                                    pass
#                        else:
#                                # pdbsaida = pdbsaida + "\n" + i
#                                pdbsaida = pdbsaida + i
#            return(pdbsaida)

    def HetatomsInChain(self,  chain):
        if self.hetatm():
            List = []
            for i in self.FileInList:
                if ("HETATM" in i) and not ("REMARK" in i):
                    #Line = filter(None, re.split(" ", i))
                    if ((not i[17:20].replace(" ", "") in List) and (chain == i[21])):
                        List.append(i[17:20].replace(" ", ""))
            return(List)
        else:
            print("excecao")

   
#al = pdb('/joao/1uij.pdb')
#al = pdb("/home/joao/IC/PDB/1uij.pdb")
#al = pdb("/home/joao/IC/1IKN.pdb")
#print al.nomedoarquivo()
#print al.Nome()
#print al.LinhasOndeTemEscritoChain()
#print al.chains()
#al = pdb("/joao/Alpha/src/teste1/models/1b8p.pdb")
#print al.ChangeHetatoms("A",   "NAG", "CO")

#print "Cadeias: ",    al.chains()
#print al.NomedaEstrutura()
#print "Heteroatomos? " ,  al.hetatm()
#print  "Agua? " ,  al.hoh()
#print "Heteroatomos: " ,  al.HetatomsInPDB()
#print al.HetatomsInChain('D')
#x = file("/joao/teste.txt",  "w")
#x.write(al.ChangeHetatoms("F",   "HOH", "CO"))
#x.close()




#data 26 de janeiro de 2016
import urllib2, os

class GetDataFromPDB(object):
	"""docstring for GetDataFromPDB"""
	def __init__(self, path, template_name):
		super(GetDataFromPDB, self).__init__()
		self.path = path
		self.template_name = template_name

	def getPDB_File(self):
		pdb_file_path = self.path + os.sep + self.__onlyTemplateName__() + ".pdb"
		response = urllib2.urlopen('http://www.rcsb.org/pdb/download/downloadFile.do?fileFormat=pdb&compression=NO&structureId=' + self.__onlyTemplateName__())
		pdb_buffer = response.read()
		pdb_file = file(pdb_file_path, "w")
		pdb_file.write(pdb_buffer)
		pdb_file.close()
		return pdb_file_path
	# http://www.rcsb.org/pdb/download/downloadFile.do?fileFormat=pdb&compression=NO&structureId=1ijk

	def __onlyTemplateName__(self):
		if len(self.template_name) == 4:
			return self.template_name
		else:
			return self.template_name[0:4]


import os

from Bio import SearchIO
from Bio import AlignIO
from Bio.PDB import PDBParser
from PirFormat import PirFormat
from modeller import *
from AlignFile import *


class AlignmentBuilder():
	def __init__(self, modeldir, templatePDBFile, alignmentfile, template_pir):
		# self.hmmr3file = hmmr3file
		self.workdir = modeldir
		self.templatePDBFile = templatePDBFile
		self.alignmentfile = alignmentfile
		self.fastaAlignment = self.__get_fasta_alignment_str__()
		self.templatePIR = AlignFile(template_pir)

	def __get_fasta_alignment_str__(self):
		fastaAlignmentFile = AlignIO.read(self.alignmentfile, "fasta") 
		fastaAlignmentList = []
		for record in fastaAlignmentFile:
			fastaAlignmentList.append(record.seq)
		return fastaAlignmentList

	def __get_target_sequence__(self):
		structureName = "seq.ali"
		structureType = "sequence"
		description = ""
		start = ""
		chain_start = ""
		stop = ""
		chain_stop = ""
		sequence = str(self.fastaAlignment[1]).upper() + self.templatePIR.get_all_heteroatoms(0)
		return PirFormat(structureName, structureType, description, start, chain_start, stop, chain_stop, sequence).get()

	def __get_template_sequence__(self):
		#env = environ()
		#mdl = model(env, file=self.templatePDBFile)
		structureName = os.path.basename(self.templatePDBFile).partition(".")[0] #melhorar isso
		parser = PDBParser()
		structure = parser.get_structure(structureName, self.workdir + "/" + self.templatePDBFile)
		model = structure[0] #get model from structure
		listOfChains = model.get_list()
		# structureType = mdl.prottyp
		# description = mdl.name
		# start = mdl.range[0].partition(":")[0]
		#chain_start = mdl.range[0].partition(":")[2]
		# stop = mdl.range[0].partition(":")[0]
		#chain_stop = mdl.range[1].partition(":")[2]
		# structureType = mdl.prottyp
		structureType = "StructureX"
		description = ""
		start = "FIRST"
		chain_start = listOfChains[0].id
		stop = "END"
		chain_stop = listOfChains[-1].id
		sequence = str(self.fastaAlignment[0]).upper() + self.templatePIR.get_all_heteroatoms(0)
		return PirFormat(structureName, structureType, description, start, chain_start, stop, chain_stop, sequence).get()

	def __get_all_heteroatoms__(self, template_pir):
		template_PirFormat = AlignFile(template_pir)
		return template_PirFormat.get_all_heteroatoms()
		

# if __name__ == '__main__':
# 	x = AlignmentBuilder("1bdm.pdb", "aligned.m", 'template.pir')
# 	alingFile = file('ali.ali','w')
# 	alingFile.write(x.__get_template_sequence__() + "\n\n") 
# 	alingFile.write(x.__get_target_sequence__())
# 	alingFile.close()
	# print x.templatePIR.get_all_heteroatoms(0)


from Modeller_Caller import modeller_caller
import os
class Malign(object):
    """docstring for Malign"""
    def __init__(self, path):
        self.path = path
        self.myscript = ""

    def create_script_in_folder(self, template, best_sequence, loop_sequence):
        script = """from modeller import *
import os

os.chdir('""" + self.path + """')
log.verbose()
env = environ()

env.io.atom_files_directory = './:../atom_files/'

aln = alignment(env)
for (code, chain) in (('""" + template + """', 'A'), ('""" + best_sequence + """', ''), ('""" + loop_sequence + """', '')):
    mdl = model(env, file=code, model_segment=('FIRST:'+chain, 'LAST:'+chain))
    aln.append_model(mdl, atom_files=code, align_codes=code+chain)

for (weights, write_fit, whole) in (((1., 0., 0., 0., 1., 0.), False, True),
                                    ((1., 0.5, 1., 1., 1., 0.), False, True),
                                    ((1., 1., 1., 1., 1., 0.), True, False)):
    aln.salign(rms_cutoff=3.5, normalize_pp_scores=False,
               rr_file='$(LIB)/as1.sim.mat', overhang=30,
               gap_penalties_1d=(-450, -50),
               gap_penalties_3d=(0, 3), gap_gap_score=0, gap_residue_score=0,
               dendrogram_file='ali.tree',
               alignment_type='tree', # If 'progresive', the tree is not
                                      # computed and all structues will be
                                      # aligned sequentially to the first
               feature_weights=weights, # For a multiple sequence alignment only
                                        # the first feature needs to be non-zero
               improve_alignment=True, fit=True, write_fit=write_fit,
               write_whole_pdb=whole, output='ALIGNMENT QUALITY')

#aln.write(file='ali.pap', alignment_format='PAP')
aln.write(file='ali.ali', alignment_format='PIR')

aln.salign(rms_cutoff=1.0, normalize_pp_scores=False,
           rr_file='$(LIB)/as1.sim.mat', overhang=30,
           gap_penalties_1d=(-450, -50), gap_penalties_3d=(0, 3),
           gap_gap_score=0, gap_residue_score=0, dendrogram_file='1is3A.tree',
           alignment_type='progressive', feature_weights=[0]*6,
           improve_alignment=False, fit=False, write_fit=True,
           write_whole_pdb=False, output='QUALITY')
"""
        script_path = self.path + os.sep + "salign.py"
        loop_script = file(script_path, "w")        
        loop_script.write(script)
        loop_script.close()
        self.myscript = script_path

    def get_model(self):
        processo = modeller_caller()
        processo.run(self.myscript)


import os
from Modeller_Caller import modeller_caller
class Loop_Refinement(object):
    """docstring for Loop_Refinement"""
    def __init__(self, initial_model, start_residue, end_residue):
            self.initial_model = initial_model
            self.start_residue = start_residue
            self.end_residue = end_residue
            self.myscript = ""
    
    def create_script_in_folder(self,path):
        script = """\
# Loop refinement of an existing model
from modeller import *
from modeller.automodel import *
import os

log.verbose()
env = environ()
os.chdir('""" + path + """')

# directories for input atom files
env.io.atom_files_directory = './:../atom_files'

# Create a new class based on 'loopmodel' so that we can redefine
# select_loop_atoms (necessary)
class MyLoop(loopmodel):
    # This routine picks the residues to be refined by loop modeling
    def select_loop_atoms(self):
        # 10 residue insertion 
        return selection(self.residue_range('""" + self.start_residue + """', ' """ + self.end_residue + """'))

m = MyLoop(env,
           inimodel='""" + self.initial_model + """', # initial model of the target
           sequence='seq.ali')          # code of the target

m.loop.starting_model= 1           # index of the first loop model 
m.loop.ending_model  = 5          # index of the last loop model
m.loop.md_level = refine.very_fast # loop refinement method; this yields
                                   # models quickly but of low quality;
                                   # use refine.slow for better models

m.make()
""" 
        script_path = path + os.sep + "loop.py"
        loop_script = file(script_path, "w")        
        loop_script.write(script)
        loop_script.close()
        self.myscript = script_path

    def __folder_of_evaluate__(self):
        pass

    def get_model(self):
        processo = modeller_caller()
        processo.run(self.myscript)
        
    def get_results(self):
        folder = os.path.dirname(self.myscript)
        mof = 0
        filemof = ""
        for files in os.listdir(folder):
            if (files.startswith('seq.ali.BL')):
                    arqr = file(folder + '/' + files, 'r')
                    arqr.readline()
                    linha = arqr.readline()
                    if (mof == 0) or (linha.partition(':      ')[2] < mof):
                            mof = linha.partition(':      ')[2]
                            filemof = folder + '/' + files
                            arqr.close()
        # if not (os.path.exists(folder + '/../loop')):
        #         os.mkdir(folder + '/../loop')
        # os.popen('cp '+ filemof +  ' ' + folder + '/../loop/')
        return filemof


class PirSequence(object):
	"""docstring for Sequence"""
	def __init__(self, user_sequence):
		self.user_sequence = user_sequence
		self.__next_sequence_symbol__ = "/"

	def number_of_chains(self):
		return self.sequence().count(self.__next_sequence_symbol__) + 1

	def sequence(self):
		list_of_lines_of_sequence = self.user_sequence.split("\n")[2:]
		str_containing_only_sequence = "\n".join(list_of_lines_of_sequence)
		return str_containing_only_sequence

	def chain(self, number_of_chain):
		chain_choosed = self.sequence().split(self.__next_sequence_symbol__)[number_of_chain - 1]
		if "*" not in chain_choosed:		
			return chain_choosed
		else:
			return chain_choosed[:-1]


	def list_heteroatoms(self, chain):
		heteroatoms = []
		for selectedResidue in self.chain(chain):
			if (selectedResidue.islower() or selectedResidue.isdigit() or not selectedResidue.isalnum()) and not (selectedResidue in ["-","*",'\n'] or selectedResidue in heteroatoms):
				heteroatoms.append(selectedResidue)
		return heteroatoms

	def change_heteroatom(self, chain, old_heteroatom, new_heteroatom):
		new_modified_sequence = self.header() + "\n"
		for selectedChain in range(1,self.number_of_chains() + 1):
			if selectedChain == chain:
				new_modified_sequence = new_modified_sequence + self.__show_chain_with_new_heteroatoms__(chain, old_heteroatom, new_heteroatom)
			else:
				new_modified_sequence = new_modified_sequence + self.chain(selectedChain)
			new_modified_sequence = new_modified_sequence + self.__next_sequence_symbol__
		new_modified_sequence = new_modified_sequence[:-1] # remove last "/"
		new_modified_sequence = new_modified_sequence + "*" # add terminator symbol
		self.user_sequence = new_modified_sequence

	def header(self):
		list_of_lines_of_header = self.user_sequence.split("\n")[0:2]
		str_containing_only_header = "\n".join(list_of_lines_of_header)
		return str_containing_only_header

	def __show_chain_with_new_heteroatoms__(self, chain, old_heteroatom, new_heteroatom):
		return self.chain(chain).replace(old_heteroatom, new_heteroatom)

	def change_het_in_position(self,heteroatom, position):
		new_modified_sequence = self.header() + "\n"
		number_of_returns = self.sequence()[:position].count("\n")
		real_position = position # - number_of_returns
		new_modified_sequence = new_modified_sequence + self.sequence()[:real_position -1] + heteroatom + self.sequence()[real_position:]
		self.user_sequence = new_modified_sequence



import os, sys 
from Modeller_Caller import modeller_caller
class mold:
    def __init__(self,file_format_pir):
        self.file_format_pir = file_format_pir

    def make_build_profilepy(self):
        script = """\
from modeller import *

log.verbose()
env = environ()

#-- Prepare the input files

#-- Read in the sequence database
sdb = sequence_db(env)
sdb.read(seq_database_file='""" + self.__installfolder__() + """/pdb/pdb_95.pir', seq_database_format='PIR', chains_list='ALL', minmax_db_seq_len=(30, 4000), clean_sequences=True)

#-- Write the sequence database in binary form
sdb.write(seq_database_file='""" + self.__installfolder__() + """/pdb/pdb_95.bin', seq_database_format='BINARY', chains_list='ALL')

#-- Now, read in the binary database
sdb.read(seq_database_file='""" + self.__installfolder__() + """/pdb/pdb_95.bin', seq_database_format='BINARY', chains_list='ALL')

#-- Read in the target sequence/alignment
aln = alignment(env)
aln.append(file='""" + self.file_format_pir + """', alignment_format='PIR', align_codes='ALL')

#-- Convert the input sequence/alignment into
#   profile format
prf = aln.to_profile()

#-- Scan sequence database to pick up homologous sequences
prf.build(sdb, matrix_offset=-450, rr_file='${LIB}/blosum62.sim.mat', gap_penalties_1d=(-500, -50), n_prof_iterations=1, check_profile=False, max_aln_evalue=0.01)

#-- Write out the profile in text format
prf.write(file='""" + os.path.dirname(self.file_format_pir) + '/build_profile.prf'"""', profile_format='TEXT')

#-- Convert the profile back to alignment format
aln = prf.to_alignment()

#-- Write out the alignment fileo
aln.write(file='""" + os.path.dirname(self.file_format_pir) + '/build_profilePIR.ali' + """', alignment_format='PIR')
aln.write(file='""" + os.path.dirname(self.file_format_pir) + '/build_profilePAP.ali' + """', alignment_format='PAP')
"""
        arq = file(os.path.dirname(self.file_format_pir) + '/build_profile.py', 'w')
        arq.write(script)
        arq.close()

    def __installfolder__(self):
        '''Returns where the AutoModel was instaled'''
        return os.path.dirname(os.path.realpath(sys.argv[0]))
#        return "/home/joaoluiz/IC/AutoModelIIH"

    def __folder_of_model__(self):
        return(os.path.dirname(self.file_format_pir))

    def find_templates(self):
        processo = modeller_caller()
        processo.run(self.__folder_of_model__() + '/build_profile.py')


from modeller import *
from modeller.scripts import complete_pdb

log.verbose()    # request verbose output
env = environ()
env.libs.topology.read(file='$(LIB)/top_heav.lib') # read topology
env.libs.parameters.read(file='$(LIB)/par.lib') # read parameters

# read model file
mdl = complete_pdb(env, 'TvLDH.B99990002.pdb')

# Assess with DOPE:
s = selection(mdl)   # all atom selection
s.assess_dope(output='ENERGY_PROFILE NO_REPORT', file='TvLDH.profile',
              normalize_profile=True, smoothing_window=15)



import subprocess
class modeller_caller:
    def __init__(self):
        self.modeller_executable = "python"
        self.process = None
    def run(self,script):
        try:
            process = subprocess.Popen([self.modeller_executable,script])
        except:
            print "Modeller or Script not found!"
        return process.wait()



from Modeller_Caller import modeller_caller
import os
class MakeProfile(object):
    """docstring for MakeProfiles"""
    def __init__(self):
        pass
    
    def create_script_in_folder(self,pdb_file):
        self.path = os.path.dirname(pdb_file)
        script = """from modeller import *
from modeller.scripts import complete_pdb
import os

os.chdir('""" + self.path + """')
log.verbose()    # request verbose output
env = environ()
env.libs.topology.read(file='$(LIB)/top_heav.lib') # read topology
env.libs.parameters.read(file='$(LIB)/par.lib') # read parameters

# directories for input atom files
env.io.atom_files_directory = './:../atom_files'

# read model file
mdl = complete_pdb(env, '""" + pdb_file + """')

s = selection(mdl)
s.assess_dope(output='ENERGY_PROFILE NO_REPORT', file='""" + os.path.basename(pdb_file)[0:-4] + """.profile',
              normalize_profile=True, smoothing_window=15)
"""
        script_path = self.path + os.sep + "get_profile.py"
        loop_script = file(script_path, "w")        
        loop_script.write(script)
        loop_script.close()
        self.myscript = script_path

    def get_model(self):
        processo = modeller_caller()
        processo.run(self.myscript)


import unittest
from should_dsl import *
from pdb import pdb


class PdbTest(unittest.TestCase):

    def setUp(self):
        self.pdb = pdb('./nescessary_files/1uij.pdb')
        self.pdb_fail = pdb('./nescessary_files/1uij_fail.pdb')

    def test_if_show_the_name_file_match_using_Nome_function(self):
        self.pdb.Nome() |should| equal_to('./nescessary_files/1uij.pdb')

    def test_if_show_the_name_file_match_using_nomedoarquivo_function(self):
        self.pdb.nomedoarquivo() |should| equal_to('./nescessary_files/1uij.pdb')

    def test_if_show_the_struct_name(self):
        self.pdb.NomedaEstrutura() |should| equal_to('1uij')

    def test_if_show_the_chains(self):
        self.pdb.chains() |should| equal_to(['A', 'B', 'C', 'D', 'E', 'F'])

#    def test_if_not_show_the_chains_if_field_CHAINS_not_found(self):
#        self.pdb_fail.chains() |should| equal_to(['A', 'B', 'C', 'D', 'E', 'F'])

    def test_the_number_of_the_first_line_that_contains_Chain(self):
        self.pdb.PrimeiraLinhaOndeTemEscritoChain() |should| equal_to(5)

    def test_if_the_template_contains_hetatms(self):
        self.pdb.hetatm() |should| equal_to(True)

    def test_if_the_template_not_contains_hetatms(self):
#        self.pdb.hetatm() |should| equal_to(True)
        pass

    def test_if_the_template_contain_HOH(self):
        self.pdb.hoh() |should| equal_to(True)

    def test_if_the_template_not_contain_HOH(self):
        pass

    def test_if_show_the_hetatms_in_PDB(self):
        self.pdb.HetatomsInPDB() |should| equal_to(['HOH'])

    def test_if_not_show_the_hetatms_in_PDB_because_not_have_hetatms(self):
#        self.pdb.HetatomsInPDB() |should| equal_to(['HOH'])
        pass
    def test_if_not_show_the_hetatms_contained_in_chains_because_not_have_hetatms(self):
        pass
    def test_if_show_the_hetatms_contained_in_chains(self):
        self.pdb.HetatomsInChain('F') |should| equal_to(['HOH'])

# parei na linha 68
if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name='automodel-server',
    version='0.1.7',
    url='https://bitbucket.org/jlalmeidaf/automodel-server-with-hmmer',
    license='',
    author='João Luiz de Almeida Filho, Jorge Hernandez Fernandez',
    author_email='joaoluiz.af@gmail.com',
    keywords='protein modeling modeller automodel',
    description=u'Server of service for structure protein prediction',
    packages=['AutomodelServerModules', 'modellerstep', 'modellingfile', 'pdb', 'tests'],
    install_requires=['argparse', 'biopython', 'rpyc', 'matplotlib'],
    scripts=['server.py']
)



import pdb, os
from Modeller_Caller import modeller_caller
import AutomodelServerModules.sequence
class Modeler:
    def __init__(self,pdb_folder,modeldir,pdb_file,ali_ali):
        self.ali_ali = modeldir + ali_ali
        self.pdb_folder = pdb_folder
        self.modeldir = modeldir
        self.seq = sequence.Sequence(self.ali_ali)
        print self.seq.structures_names()
        self.structures = str(self.seq.structures_names()[0:-1])[1:-1]

    def make_get_model_py(self):
        script = """\
from modeller import *
from modeller.automodel import *    # Load the automodel class
import os
os.chdir('""" + self.__folder_of_model__() + """')

log.verbose()

env = environ(rand_seed=-12312)  # To get different models from another script
# directories for input atom files
env.io.hetatm = env.io.water = True


env.io.atom_files_directory = ['""" + self.modeldir + """']
a = automodel(env,
              alnfile='""" + self.ali_ali + """',      # alignment filename (ali.ali)
              knowns=(""" + self.structures + """),             # codes of the templates (variavel da est)
              sequence='seq.ali',           # code of the target (seq.ali)
              assess_methods=assess.GA341)  # request GA341 assessment
a.starting_model= 1                 # index of the first model
a.ending_model  = 5                 # index of the last model
                                    # (determines how many models to calculate)
a.deviation = 4.0                   # has to >0 if more than 1 model

a.make()                            # do homology modelling"""
        arq = file(self.modeldir + 'get_model.py', 'w')
        arq.write(script)
        arq.close()

    def __folder_of_model__(self):
        return self.modeldir

    def model_sequence(self):
        processo = modeller_caller()
        processo.run(self.__folder_of_model__() + 'get_model.py')

    def get_results(self):
        folder = self.modeldir
        mof = 0
        filemof = ""
        for files in os.listdir(folder):
            if (files.startswith('seq.ali.B')):
                    arqr = file(folder + '/' + files, 'r')
                    arqr.readline()
                    linha = arqr.readline()
                    if (mof == 0) or (linha.partition(':      ')[2] < mof):
                            mof = linha.partition(':      ')[2]
                            filemof = folder + '/' + files
                            arqr.close()
        # if not (os.path.exists(folder + '/../results')):
        #         os.mkdir(folder + '/../results')
        # os.popen('cp '+ filemof +  ' ' + folder + '/../results/')
        return filemof


import pylab
import modeller

def get_profile(profile_file, seq):
    """Read `profile_file` into a Python array, and add gaps corresponding to
       the alignment sequence `seq`."""
    # Read all non-comment and non-blank lines from the file:
    f = file(profile_file)
    vals = []
    for line in f:
        if not line.startswith('#') and len(line) > 10:
            spl = line.split()
            vals.append(float(spl[-1]))
    # Insert gaps into the profile corresponding to those in seq:
    for n, res in enumerate(seq.residues):
        for gap in range(res.get_leading_gaps()):
            vals.insert(n, None)
    # Add a gap at position '0', so that we effectively count from 1:
    vals.insert(0, None)
    return vals

e = modeller.environ()
a = modeller.alignment(e, file='TvLDH-1bdmA.ali')

template = get_profile('1bdmA.profile', a['1bdmA'])
model = get_profile('TvLDH.profile', a['TvLDH'])

# Plot the template and model profiles in the same plot for comparison:
pylab.figure(1, figsize=(10,6))
pylab.xlabel('Alignment position')
pylab.ylabel('DOPE per-residue score')
pylab.plot(model, color='red', linewidth=2, label='Model')
pylab.plot(template, color='green', linewidth=2, label='Template')
pylab.legend()
pylab.savefig('dope_profile.png', dpi=65)
